#!/usr/bin/env python
#<title>Childsplay.py_0.69</title>#
"""
     Copyright (C) 2002-2003 Stas Z. <stas@linux.isbeter.nl>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 Childsplay - A suite of educational games for young children.

  This is beta software, please send bugs and/or comments to
  childsplay@users.sourceforge.net"""

import sys,os

# Check for pygame
try:
    import pygame
except ImportError,info:
    print >> sys.stderr,info,"\n\n Childsplay depends on pygame."
    print >> sys.stderr, "\n You must first install the pygame package.\n Due to the fact that this package has some tricky\n dependencies, you better install the one that came with your distribution."
    sys.exit(1)


# Check for assetml utils
try:
    import pyassetmlSDL
    #assetmlSDL = import_module(os.path.join(basedir,'pyassetmlSDL.py'))
except (ImportError,MyError),info:
    print >> sys.stderr,info,"\n\n This version of childsplay depends on pyassetmlSDL"
    print >> sys.stderr, "\n You can get the pyassetml package at:","\n http://www.ofset.org/assetml"
    sys.exit(1)



DEBUG=0

## Global options, config dic. Look in def get_rc() for format ##
#__name__ = just to be consistent with a real config file
#fullscr = 1 #set to toggle fullscreen
rcdic_defaults = {'default':{'__name__':'default','fullscr':1}}

# moved to version.py
version = '0.69' # Must be in here, used by the plugin install script

## Global paths ##
basedir = os.path.dirname(sys.argv[0])
libdir = os.path.join(basedir,'lib')
rcdir = os.path.join(libdir,'ConfigData')
localedir = os.path.join('/','usr','share','locale')


## Global flags ##
MONITOR = None # Special monitor plugin, optional, future use
RCFILES = None # Can we use configuration files
   
helptext = "Not much help here yet, look at the manpage and docs."
## end globals ###


if DEBUG: print "sys.argv",sys.argv
if len(sys.argv) > 1:
    for item in sys.argv[1:]:
        if item == '--fullscreen':
            fullscr = 1
        elif item == '--window':
            fullscr = None
        elif item == '--version':
            print version
            raise SystemExit
        elif item == '--help':
            print helptext
            raise SystemExit
    
if not DEBUG: print >> sys.stderr,__doc__

# Start splash screen, runs in a seperate thread
try:
    import splash
except (RuntimeError,pygame.error),info:
    print info

# Start CP   
try:
    import random,fnmatch,gettext,locale,pickle
    from pygame.constants import RLEACCEL,KEYDOWN,K_BACKSPACE,K_RETURN,MOUSEBUTTONDOWN,QUIT,K_ESCAPE,K_F1,FULLSCREEN
    from utils import *
except ImportError, info:
    print >> sys.stderr, "Can\'t import module ",info
    trace_error()
    raise SystemExit

pygame.init()
 



# Check if there's a childsplay dir in the home directory
homedir = os.path.join(os.environ['HOME'],HOME_DIR_NAME)
if not os.path.exists(homedir):
    tree = HOME_DIR_TREE.replace(';',os.sep)
    for piece in tree.split(','):
        piece = os.path.join(homedir,piece)
        if not os.path.exists(piece):
            try:
                os.makedirs(piece)
            except OSError,info:
                print >> sys.stderr, "Message while creating directory tree in",os.environ["HOME"],"\n",info
            
## check and get rc parser object
try:
    RcParser_module = import_module(os.path.join(basedir,'ConfParser.py'))
except ImportError, info:
    print >> sys.stderr, "Can\'t import module ",info
    trace_error()
    print >> sys.stderr,"Using configuration defaults"
else:
    RCFILES = 1

def get_rc(file):
    """ Loads and pars a config file and puts it in a dic.
    The dic is a dic of dic's;
     {'default':{'__name__':'default','option1':'value'},'section':{'__name__'...}}.
     file = name of the module rcfile, without the path.
    """
    rcobj = RcParser_module.ConfParser()
    try:
        rcobj.read(os.path.join(libdir,'ConfigData',file))
    except RcParser_module.ParsingError:
        trace_error()
        return None
    
    rcdic = {}
    for section in rcobj.sections():
        optdic = {}
        for option in rcobj.options(section):
            optdic[option] = rcobj.get(section,option)
        rcdic[section] = optdic
    if DEBUG: print "rcdic.values()",rcdic.values()
    if rcdic and rcdic.has_key('default') and len(rcdic.keys()) > 1:# update sections with the default section
        new_dic ={}
        dic_defaults = rcdic['default'].copy()
        del rcdic['default']
        for dic in rcdic.values():
            #print 'dic old',dic
            d = dic_defaults.copy()
            d.update(dic)
            #print 'd ',d
            new_dic[dic['__name__']] = d.copy()
        if DEBUG: print "new_dic ",new_dic
        return new_dic
    return rcdic
    
def set_globalrc(dic):
    global rcdic,fullscr
    try:
        fullscr = int(rcdic['default']['fullscr']) # Nothing else at the moment
    except (KeyError,ValueError,TypeError),info:
        print >> sys.stderr," Problems reading the childsplay config file,\n",\
                info,"\n using the defaults"
        rcdic = rcdic_defaults
        set_globalrc(rcdic) # read again
    
## look for childsplay rc file,XXX these options can be overruled by the commandline args.
if RCFILES:
    rcdic = get_rc('childsplayrc')
else:
    rcdic = rcdic_defaults
set_globalrc(rcdic)
## Overrule rc valeus, this sucks. This must be rewritten and cleaned up
## XXX we should get a nice and clean way to handle config files 
if len(sys.argv) > 1:
    for item in sys.argv[1:]:
        if item == '--fullscreen':
            fullscr = 1
        elif item == '--window':
            fullscr = None
#print "rcdic",rcdic

## i18n ##
stdloc = locale.setlocale(locale.LC_ALL,'')
try:
    gettext.install('childsplay',localedir)
except IOError,info:
    trace_error()
    print >> sys.stderr, "Can't find i18n files for",stdloc
    print >> sys.stderr, "Using standard, -> english"
    import __builtin__
    __builtin__.__dict__['_'] = gettext.gettext

### Strings that are used in this module ask_exit function are translated
### here because when a game is played the gettext is bind to the
### game mo file. When the user hits escape/close window from within the 
### game the ask_exit function is called by the conroller, but the i18n text from
### controller lives in the childsplay mo file and is never found by gettext.
ASK_EXIT_TXT = [_("Do you really want to quit ?"),_("Hit (Y)es or (N)o")]
ASK_EXIT_YES = _("y")

# check if we have menu icons for the found games
modules = []
for name in fnmatch.filter(os.listdir(libdir),'*.py'):
    if DEBUG: print >> sys.stderr, "plugin",name
    if os.path.exists(os.path.join(basedir,'Data','icons',name[:-3]+'.icon.png')):
        modules.append((name[:-3]))
    else:
        print >> sys.stderr,'Can\'t find menu icon for',name,'\nRemoving',name,'from modules list'
    if DEBUG: print "gamemodules",modules   
## setup a dic; key=menu icon pos., value = (module name, icon name). This dic is used to display
## the menu icons and as a lookup table for the corresponding modules.
menudic = {}
rows = (len(modules) < 6)+len(modules)/6
y = 80
x = 80
for row in range(rows):
    for name in modules:
        menudic[(x,y,80,80)] = (name,name+'.icon.png',1)
        x += 100
    y += 100
    x = 80


## Namespace containers, filled at run time
class Img:
    pass

class Snd:
    pass

class Misc:
    pass
    
# create two assetmlSDL instances one for parsing images and one for sounds
Assets_img = pyassetmlSDL.AssetmlSDL()
Assets_img.set_mldir('childsplay/childsplay-images/childsplay-images.assetml')

Assets_snd = pyassetmlSDL.AssetmlSDL()
Assets_snd.set_mldir('childsplay/childsplay-sounds/childsplay-sounds.assetml')   

########### End toplevel code #####################


############# XXX not in use yet #####################
class Mouse:
    """ Not in use"""
    def __init__(self):
        start = pygame.display.get_surface().get_size()
        self.pointer = load_image(os.path.join(basedir,'Data','pointer.png'),1)
        self.rect = self.pointer.get_rect()
        self.rect.move_ip(300,300)
                
    def update(self):
        pos = pygame.mouse.get_pos()
        self.rect.centerx,self.rect.centery = pos
        return 0
########## End unused piece ######################################

class Firework:
    """ Class used to display the fireworks at game-over time."""
    def __init__(self,x,letter):
        self.sndboom = Snd.boom
        
        self.fireR = Img.firework1
        self.fireL = Img.firework2
        self.img = self.fireR
        self.letter = letter
        self.x = x
        self.y = 600
        self.speed = 10
        self.height = 100
        self.rect = self.fireL.get_rect()
        self.rect.move_ip(self.x,self.y)
        
    def update(self):
        """ To be called from the eventloop, when the height is reached the img attr. is
          changed to the letter attr. This way the eventloop is displaying the letter"""
        if self.y <= self.height:
            self.y = 600
            self.sndboom.play()
            self.img = self.letter
            return 1
        else:
            self._move()
        return 0
    
    def _move(self):
        """ Move the rocket to thr top of your monitor.
          Each time switching between two images to give the impression of fire"""
        self.y -= self.speed
        self.rect.move_ip(0,-self.speed)
        if self.img is self.fireR:
            self.img = self.fireL
        else:
            self.img = self.fireR

class Menu:
    """ Menu class, mimics a game module so it can be used just like any other plugin.
      From the controllers point of view it's just an other plugin.
      This class has everything needed to run the menu"""
    def __init__(self,screen,backcol,menudic):
        self.screen  =screen
        self.backcol = backcol
        self.menudic = menudic
        
        self.menuitems = []
        self.menurects = []
        self._setup()
                
    def _setup(self):
        class MenuItem:
            """ Container, becomes a object for each game (icon, position, modulename)"""
            pass
        
        ## create menu objects ###
        for key in self.menudic.keys():
            obj = MenuItem()
            obj.img = load_image(os.path.join(basedir,'Data','icons',self.menudic[key][1]),self.menudic[key][2])
            if DEBUG: print >> sys.stderr, "menu object image",obj.img
            r = obj.img.get_rect()
            obj.rect = r.move(key[:2])
            self.menuitems.append((obj))
            self.menurects.append((obj.rect))
            
    def show(self):
        """ Blit the menu objects on the main screen"""
        dirtyrects = []
        for obj in self.menuitems:
            dirtyrects.append((self.screen.blit(obj.img,obj.rect)))
        if DEBUG: print >> sys.stderr,"Menu.show rects",dirtyrects
        pygame.display.update(dirtyrects)
    
    def helptitle(self):
        """ Used when the user hits the ? in the menu screen"""
        return "Childsplay"    
    
    def help(self):
        text = [_("A suite of educational games for young children."),
_("It's using a plug-in system for the games, so childsplay is basically a"),
_("framework where you put in as many games as you like."), 
_("Childsplay comes with a few standard games, so you might want to check"),
_("the website (see below) for additional games."),
" ",
_("All the games have there own help page, just start a game and hit the"),
_("questionmark again"),
" ",
_("Childsplay and all the games are licensed with the GNU-GPL."),
_("All the sounds and pictures as well as the fonts used by childsplay and the games"),
_("are free in the public domain or used with permission of the owner."),
_("If you think that something is used without permission or you have a "),
_("'See-you-in-court' copyright on it, please let me know and i will remove it"),
" ",
_("For updates and info: http://childsplay.sourceforge.net"),
_("For additional game-modules : http://childsplay.sourceforge.net/plugins.html"),
_("Comments, bugs, ideas and other things of interest, send email to:"),
_("childsplay@users.sourceforge.net"),
" ",
_("Credits:"),
"TrueType font by www.larabiefonts.com (freeware ttf's)",
"Configuration file parser by Charles Cazabon."
" ",
"Thanks:",
"French translation by jp.ayanides@free.fr",
"Danish translation by Gunner, gp@lyngbjerggaardskolen.dk",
"German translation by Bernd, info@herdsoft.com",
"Turkish translation by Cartan, c_artan@hotmail.com",
"Spanish translation by Adrian, adrianmatematico@yahoo.com.mx"]
        
        return text
    
    def loop(self,events):
        """ The menu's own eventloop, called by the controller mainloop
          Returns the users choice as a dic key from the menudic."""
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                for r in self.menurects:
                    if r.contains(pos):
                        Snd.music1.stop()
                        return tuple(r),0
        return 0,0
      
class ScoreThing:
    """ Provides a score surface, the thing you see to the right."""
    def __init__(self,fsize=12,fcol=(0,0,0)):
        self.fsize = fsize
        self.fcol = fcol
        surf_s,s_size = font2surf('Score:',self.fsize,self.fcol,None,1)
        self.score_pos = (12,4+s_size[1])
        surf_size = (s_size[0]+12,s_size[1]*2+4)
        self.base_surf = pygame.Surface(surf_size).convert()
        pygame.draw.rect(self.base_surf,(0,255,0),self.base_surf.get_rect(),4)
        self.base_surf.blit(surf_s,(6,4))
        
    
    def update(self,score='0000'):
        """ Called from the controller when there's a score to add. 
          Rteurns a updated sdl surface."""
        surf = self.base_surf.convert()
        sc = ('0000'+score)[-4:]# left fill with zero's
        surf_s,s_size = font2surf(sc,self.fsize,self.fcol,None,1)
        surf.blit(surf_s,self.score_pos)
        return surf
        
class Score:
    """ High score stuff, takes care of everything that's high-score related.
     It uses a 'scoresheet' which is a dic with lists as values and the game names
      as keys and is stored on disk as a 'pickle object' to get a cheap encrytion."""
    def __init__(self):
        self.hsc_ttf = os.path.join(basedir,'Data','bullpen3.ttf')
        self.sc_ttf = os.path.join(basedir,'Data','bluebold.ttf')
        self.sc_fsize = 32
        self.sc_fcol = (7,249,23)
        """self.bal_l_img = load_image(os.path.join(basedir,'Data','balloonsl.png'),1)
        self.bal_r_img = load_image(os.path.join(basedir,'Data','balloonsr.png'),1)
        """
        self.bal_l_img = Assets_img.get_assets(('balloonsl.png',))# This way get_assets returns one object
        self.bal_r_img = Assets_img.get_assets(('balloonsr.png',))
        try:
            f = open(os.path.join(basedir,'Data','score'),'r')
            self.score_sheet = pickle.load(f)
            f.close()
        except (EOFError,IOError),info:
            trace_error()
            print >> sys.stderr,'Something went wrong with the score file\n Trying to create a new one'
            self.score_sheet = self._make_default_sheet()
            try:
                self._put_sheet()
                print >> sys.stderr, 'OK'
            except MyError:
                print >> sys.stderr,'Problems with the scoring class, using bogus scoring',\
                info,'\n',MyError.name,MyError.line,'\nPlease send a bug report with the above stuff to\n childsplay@users.sourceforge.net'

    
    def _make_default_sheet(self):
        """ Used if there's no score file found, this generaly means that someone deleted the
         score file. (why should you delete it anyway?).
         You can run childsplay as root and this will create a new score file.
         SECURITY-WARNING, childsplay should __never__ be run as root, so just start childsplay
         and stop it, the score file would be created at startup."""
         
        sheet = {}
        templ = []
        for name in ['Tux','Linus','Guido','RMS','Dijkstra']:
            templ.append(((100,name)))
        templ.sort()
        for name in modules:
            sheet[name.capitalize()] = templ[:]
        return sheet

    def _fetch_score(self):
        """ Load the high score file, makes a default score sheet when the load fails."""
        try:
            f = open(os.path.join(basedir,'Data','score'),'r')
            score_sheet = pickle.load(f)
            f.close()
        except (IOError,EOFError):
            trace_error()
            print >> sys.stderr,'Error in score class, don\'t use it','Using bogus sheet'
            return self._make_default_sheet()
        return score_sheet
            
    def _put_sheet(self):
        """ Saves the high-score to disk, prints errors when it fails but continue anyway"""
        try:
            f = open(os.path.join(basedir,'Data','score'),'w')
            pickle.dump(self.score_sheet,f)
            f.close()
        except IOError,info:
            trace_error()
            print >> sys.stderr,'Error in writing score'
            MyError.name = 'Error in writing score'
            MyError.line = info
            #raise MyError
    
    def get_score(self,name):
        """ First get the file from disk (_fetch_score), then get the
         list representing the high-score for this game (name)
         If there's no entry returns a messages, otherwise return
         the list."""
        self.score_sheet = self._fetch_score()
        #print >> sys.stderr,"score name",name
        #print >> sys.stderr,"score sheet",self.score_sheet
        try:
            sheet = self.score_sheet[name]
        except KeyError:
            return [(0,'Not possible to score points here')]
        sheet.sort()
        sheet.reverse() #list items -> (score,name)
        return sheet

    def check_high_score(self,name,score):
        """ Check if the score is a high-score. If true set high-score flag and start _high-score""" 
        sheet = self.get_score(name)
        
        flag = None # Not in use, yet
        if score >= sheet[-1][0]:
            flag = 1
            if len(self.score_sheet[name]) >= 8:
                self.score_sheet[name] = self.score_sheet[name][:7]
                self._put_sheet()
            self.score_sheet[name] = self._high_score(name,score)
            self._put_sheet()
        self.clear_scr()
        return flag #trigger game over screen or not. Not in use, yet
    
    def _get_key(self):
        """ Basic edeting function, used in the 'give your name'"""
        # REMINDER, change this in using the new editing function from utils.py
        event = pygame.event.poll()
        if event.type is KEYDOWN:
            if 96<event.key<123 :
                surf,spam = font2surf(chr(event.key),self.sc_fsize,self.sc_fcol,self.hsc_ttf)
                return surf,chr(event.key)
            elif event.key is K_BACKSPACE:
                return None,'back'
            elif event.key is K_RETURN:
                return None,'end'
        return None,None
    
    def _high_score(self,name,score):
        """ This starts the whole high-score event, usses it's own eventloop and
         takes care of everything"""
        sheet = self.get_score(name)
        self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
        temp_back = pygame.Surface((800,500))
        temp_back.blit(self.screen,(0,0),(0,0,800,500))
        #self._score_eyecandy(temp_back)
        pygame.display.update(self.screen.blit(self.bal_l_img,(0,0)))
        pygame.display.update(self.screen.blit(self.bal_r_img,(665,0)))
        
        #file = os.path.join(basedir,'Data','Winner2.ogg')
        #load_music(file).play()
        Assets_snd.get_assets(('Winner2.ogg',)).play(1)
                
        txt = _("Congratulations,")
        surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
        x = 150
        y=20
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = _("You are a winner")
        surf,spam = font2surf(txt,32,self.sc_fcol,self.hsc_ttf)
        x = 200
        y=120
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = str(score)+_(" Points !!")
        surf,spam = font2surf(txt,48,self.sc_fcol,self.hsc_ttf)
        x = 200
        y=200
        pygame.display.update(self.screen.blit(surf,(x,y)))
        
        txt = _("Please give your name: ")
        surf,spam = font2surf(txt,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
        x = 20
        y=350
        pygame.display.update(self.screen.blit(surf,(x,y)))
        x += surf.get_width()
        offset = 0
        text = ''
        surf = None
        temp_back.blit(self.screen,(0,0),(0,0,800,500))
        while len(text) < 10:
            surf, key = self._get_key()
            if not surf and not key: continue
            #print key
            if key != 'end' and key != 'back':
                old_pos = self.screen.blit(surf,(x+offset,y))
                pygame.display.update(old_pos)
                offset += surf.get_width()
                text = text+key
            elif key == 'back' and len(text) > 0:
                text = text[:-1]
                if len(text) != 0:
                    surf,spam = font2surf(text,self.sc_fsize,self.sc_fcol,self.hsc_ttf)
                    offset = surf.get_width()
                else:
                    offset = 0
                    pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
                    continue
                #print x,y,temp_back
                pygame.display.update(self.screen.blit(temp_back,(x,y),(x,y-4,300,100)))
                pygame.display.update(self.screen.blit(surf,(x,y)))
            elif key == 'end':
                break
        #print text
        sheet.append(((score,text)))
        sheet.sort()
        sheet.reverse()
        return sheet
        
class Contr(Score):
    """ The main controller of the program is in charge of everything, besides some things
     that are in the childsplay.main(). (please look there also).
     When a game is started by the main function control is turnt over to this controller
     Now i'm become the borg-queen, the controller of worlds :-)"""
     
    def __init__(self):
                       
        ## Set icon for the window manager ##    
        file = os.path.join(basedir,'Data','chpl-icon-48.xpm')
        try:
            surface = pygame.image.load(file)
        except pygame.error:
            trace_error()
            print >> sys.stderr,'Could not load image "%s"\n %s'%(file, pygame.get_error())
        surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
        pygame.display.set_icon(surface)
               
        self.tuxplace = 20
        self.tuxrects = []
        self.mainmenurects = []
        self.setup()
        # fire up the score stuff
        Score.__init__(self)
                    
    def setup(self):
        x = 800
        y = 600
        self.y = y
        self.x = x
        if fullscr:
            self.screen = pygame.display.set_mode((x, y),FULLSCREEN)
        else:
            self.screen = pygame.display.set_mode((x, y))
        self.sdim = Dimmer(keepalive=0)
        self.clock = pygame.time.Clock()
        
        pygame.display.set_caption(_('Childsplay, educational games for young children'))
        self.backcol = (80,151,244)
        self.ttf = os.path.join(basedir,'Data','bluehigh.ttf')
        self.backgr = pygame.Surface((x,y))
        
        files = ('land.png','stop.png','question1.png','hscore.png','tux.png',\
                'OK_but.png','MORE_but.png','firework1.png','firework2.png')
        Assets_img.get_assets(files,Img)
        if DEBUG: print Img.__dict__
        
        Assets_img.set_transparent_image(0)
        Img.blue = Assets_img.get_assets(('blue.jpg',)) 
        #Img.land = Assets_img.get_assets(('land.png',))
        Assets_img.set_transparent_image(1)
        
        self.backgr.blit(Img.blue, (0,0))
        self.backgr.blit(Img.land, (0,y-86))
            
        self.mainmenurects.append((self.backgr.blit(Img.stop, (690,y-95))))
        self.mainmenurects.append((self.backgr.blit(Img.question1, (32,y-96))))
        self.mainmenurects.append((self.backgr.blit(Img.hscore, (120,y-85))))
        Img.black = pygame.Surface(Img.firework1.get_size()).convert()
        
        
        """
        Img.back = load_image(os.path.join(basedir,'Data','blue.jpg'),0)
        self.backgr.blit(Img.back, (0,0))
        
        # Build screen
        Img.land = load_image(os.path.join(basedir,'Data','land.png'),1)
        self.backgr.blit(Img.land, (0,y-89))
        Img.stop  = load_image(os.path.join(basedir,'Data','stop.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.stop, (700,y-90))))
        Img.qmark  = load_image(os.path.join(basedir,'Data','question1.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.qmark, (32,y-96))))
        Img.hscore = load_image(os.path.join(basedir,'Data','hscore.png'),1)
        self.mainmenurects.append((self.backgr.blit(Img.hscore, (120,y-85))))
        
        Img.fireR = load_image(os.path.join(basedir,'Data','firework1.png'),1)
        Img.fireL = load_image(os.path.join(basedir,'Data','firework2.png'),1)
        
        
        Img.tux = load_image(os.path.join(basedir,'Data','tux.png'),1)
        img.OK-but = load_image(os.path.join(basedir,'Data','OK-but.png'),1)
        Img.MORE = load_image(os.path.join(basedir,'Data','MORE-but.png'),1)
        """
        
        
        
        self.scorething = ScoreThing(24,(0,255,0))
        self.scorething_place = (600,y-60)
        self.backgr.blit(self.scorething.update(),self.scorething_place)
        
        self.screen.blit(self.backgr, (0, 0))
        Img.screen = self.screen
        
        pygame.display.update()
        
        #self.mouse = Mouse()
        """
        file = os.path.join(basedir,'Data','level_clear.wav')
        #Snd.levelclear = load_sound(file) 
        
        
        # Stuff for the fireworks class (game over)
        file = os.path.join(basedir,'Data','boom.wav')
        Snd.boom = load_sound(file)
        file = os.path.join(basedir,'Data','rocket.wav')
        Snd.rocket = load_sound(file)
        """
        files = ('level_clear.wav','boom.wav','rocket.wav')
        Assets_snd.get_assets(files,Snd)
        
        
        
    def mainloop(self,obj):
        """ This is the main eventloop (M.O.A.L - mother of all loops),from this loop the game loop
         is called. The M.O.A.L. gets the events list, calls the game loop, monitors the return values
         and takes appropiate actions."""
        
        stop,score = 0,0
        self.score = 0
        # stop and score flags
        # stop -> 0=continue, 2=user exit (hit the stopsign), -1=clear screen, 1=exit (game finished)
        while stop == 0:
            self.clock.tick(40)
            pygame.event.pump()
            events = pygame.event.get()
            
            #######  Not used yet  ###################################
            #old_pos = self.screen.blit(self.backgr,self.mouse.rect,self.mouse.rect)
            #self.mouse.update()
            #r = self.screen.blit(self.mouse.pointer,self.mouse.rect)
            #pygame.display.update((old_pos,r))
            #########################################################
            
            ### call game loop ##
            stop,score = obj.loop(events[:])
            ## checks for return values
            if score:
                #print 'c.mainloop stop',stop,'score',score
                self.score += score
                if game_score >= 0 and self.score > -1:
                    self.display_scorething(str(game_score+self.score))
                score = 0
            for event in events:
                if event.type is QUIT:
                    self.ask_exit()
                elif event.type is KEYDOWN and event.key is K_ESCAPE:
                    self.ask_exit()
                # THIS IS ONLY FOR TAKING SCREENSHOTS
                # IT SHOULD BE COMMENT OUT FOR RELEASE
                #elif event.type is KEYDOWN and event.key == K_F1:
                #    try:
                #        scr = pygame.display.get_surface()
                #        print scr
                #        pygame.image.save(scr,'/tmp/sdlscreen.BMP')
                #    except RuntimeError,info:
                #        print >> sys.stderr, info
                # END OF SCREENSHOT STUFF
                elif event.type is MOUSEBUTTONDOWN:
                    pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                    for r in self.mainmenurects:
                        if r.contains(pos):
                            #print r.topleft
                            if r.topleft[0] == 32:
                                stop = self.get_help(obj)
                            elif r.topleft[0] == 120 and obj.__class__.__name__ == 'Game':
                                #OK show score
                                stop = self.show_score(Misc.helptitle)
                            elif r.topleft[0] == 690:
                                if obj.__class__.__name__ == 'Menu':
                                    stop = self.ask_exit()# if return 0 then no exit    
                                else:
                                    stop = 2
        return stop,self.score # let the main function decide what next
        
    def ask_exit(self):
        """ Really quit ?, the key to hit is also i18n. The strings are translated
          in the toplevel code because we can get here from within a game.
          look in the toplevel for remarks."""
        oldclip = self.screen.get_clip()
        self.screen.set_clip()
        text = ASK_EXIT_TXT
        dirtyrects = []
        self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
        #sdim.dim(darken_factor=200,color_filter=(9,247,19)) # Used for testing
        
        x= 100
        y= 200
        fcol = (255,0,0)
        #fcol = (224,109,67) # Used for testing
        for line in text:
            surf,size = font2surf(line,56,fcol,self.ttf)
            dirtyrects.append((self.screen.blit(surf,(x,y))))
            y += surf.get_height() + 50
        pygame.display.update(dirtyrects)
        
        stop = 0
        while stop == 0:
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key == ord(ASK_EXIT_YES.lower()):
                    raise SystemExit
                elif event.type is KEYDOWN and not event.key == ord(ASK_EXIT_YES.lower()):
                    stop = 1
        self.sdim.undim()
        self.screen.set_clip(oldclip)
        return 0
            
    def display_scorething(self,score):
        """ Blit the score display to the main screen"""
        if score < 0: return
        self.screen.set_clip()
        self.backgr.set_clip()
        surf = self.scorething.update(score)
        pygame.display.update(self.screen.blit(surf,self.scorething_place))
        self.backgr.blit(surf,self.scorething_place)
        self.screen.set_clip(0,0,800,500)
        self.backgr.set_clip(0,0,800,500)
            
    def show_score(self,name):
        """ Display a hall-of-fame, this uses methods from the Score class and the help method.
        This is called when the user hits the hall-of-fame icon"""
        class spam:
            """ Bogus class, the help method expect a class obj"""
            def __init__(self,name,txt):
                self.name = name
                self.txt = txt
            def helptitle(self):
                return self.name
            def help(self):
                return self.txt
        txt = []
        i = 1
        for item in self.get_score(name):
            txt.append(("               %s  %s%s%s" % (i,item[1]," - ",item[0])))
            i += 1
        obj = spam(name,txt)
        self.get_help(obj,32)
        return 0
    
    def clear_scr(self,backgr=None):
        self.screen.blit(screendim.buffer,(0,0))
        if backgr:
            self.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
    
    def place_tux(self):
        """ Place a tux when a level is cleared, the wait is needed to let the music finish"""
        Snd.level_clear.play()
        screendim.undim()
        for r in self.tuxrects:
            pygame.display.update(self.screen.blit(Img.tux, r))
        self.tuxplace = self.tuxplace+ 80
        r = self.screen.blit(Img.tux, (120 + self.tuxplace,self.y-94))
        self.backgr.blit(Img.tux, (120 + self.tuxplace,self.y-94))
        self.tuxrects.append((r))
        pygame.display.update(r)
       
        pygame.time.wait(1500)
    
    def get_help(self,obj,fsize=26):
        """ Display the help text for the current game/plugin. It's using it's own eventloop.
          The text must be provided by the plugin."""
        
        if not obj.help():
            return 0
        ## cols and maxlines must be changed in conjuction with the fontsize 
        cols = 86
        maxlines = 13
        
        helptxt = txtfmt(obj.help(), cols)
        
        helptitle = obj.helptitle()
        oldclip = self.screen.get_clip()
        self.screen.set_clip()
                
        lines = len(helptxt)
        if DEBUG: print "get_help lines",lines
        for i in range(0, lines, maxlines):
            self.sdim.undim()
            self.sdim.dim(darken_factor=180,color_filter=(51,10,10))
            update_rects = []
            part = helptxt[i:i+maxlines]
            surf = txt2surf(part,self.x-20,self.y-104,fsize,helptitle,1,(158,196,107),self.ttf)
            update_rects.append((self.screen.blit(surf,(0,0))))
            if i+maxlines < lines: #more to come
                okrect = self.screen.blit(Img.OK_but,(self.x-700,self.y-90))
                update_rects.append((okrect))
                morerect = self.screen.blit(Img.MORE_but,(self.x-400,self.y-90))
                update_rects.append((morerect))
            else:
                okrect = self.screen.blit(Img.OK_but,(self.x-500,self.y-90))
                update_rects.append((okrect))
                morerect = None
            pygame.display.update((update_rects))
        
            # go in eventloop and wait for user action        
            stop = 0
            while stop == 0:
                events = pygame.event.get()
                for event in events:
                    if event.type is QUIT:
                        self.ask_exit()
                    elif event.type is KEYDOWN and event.key is K_ESCAPE:
                        self.ask_exit()
                    
                    if event.type is MOUSEBUTTONDOWN:
                        pos = pygame.Rect(pygame.mouse.get_pos() + (4,4))
                        if okrect.contains(pos):
                            self.sdim.undim()
                            self.screen.set_clip(oldclip)
                            return 0
                        elif morerect:
                            if morerect.contains(pos):
                                stop = 1
                                break
        return 0
        
    def game_over(self):
        """ The game-over screen, restores the screen to the begin situation"""
        
        # reset counters for placing tux
        self.tuxplace = 0
        self.tuxrects = []
        self.gameoverobj = []
        x = 10
        for item in 'GAME OVER':# can be any text
            letter,size = font2surf(item,96,(209,249,9),self.ttf)
            self.gameoverobj.append((Firework(x,letter)))
            x = x + size[0][1]-16
        
        self.sdim.dim(darken_factor=255,color_filter=(0,0,0))
        for obj in self.gameoverobj:
            Snd.rocket.play()
            stop = 0
            while stop == 0:
                self.clock.tick(60)
                r = self.screen.blit(Img.black,obj.rect)
                stop = obj.update()
                rr = self.screen.blit(obj.img,obj.rect)
                pygame.display.update((r,rr))
            pygame.time.wait(500)
        pygame.time.wait(1000)
        self.sdim.undim()
        self.clear_scr(1)     

def main():
    """ Main function responsable for loading and calling the start methods of the games,
     and after some init stuff turns control over to the Controller (class Contr)"""
    global screendim,game_score
    
    c  = Contr()# Start master control
    
    modul = 'Menu'
    screendim = Dimmer(keepalive=1) # dimmer is used for screen copying
    screendim.dim(darken_factor=0) # just stores a copy in Dimmer.buffer
    screen  = c.screen
    backcol = c.backcol
    menu = Menu(screen,backcol,menudic)# for the first time call menu here
    
    while 1:
        #file = os.path.join(basedir,'Data','music1.ogg')
        #Snd.music = load_music(file)
        Assets_snd.get_assets('music1.ogg', Snd)
        Snd.music1.play(-1)
        
        menu.show()
        choice,spam = c.mainloop(menu)
        plugin_name = menudic[choice][0]
        if choice == 2:
            raise SystemExit
        #print menudic[choice]
        c.screen.blit(screendim.buffer,(0,0))
        c.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
        
        stop = None
        try:
            # import game module
            game_module = import_module(os.path.join(libdir,plugin_name+'.py'))
                        
            try:
                if game_module.RCFILE:# can we use a rcfile?
                    plug_rcdic = get_rc(plugin_name+'rc')
                else:
                    plug_rcdic = {}
            except (AttributeError,RuntimeError),info:
                print >> sys.stderr,"Error while reading rc file.\n Maybe you should upgrade the plugins >= 0.68 ",info
                trace_error()
                plug_rcdic = {}
                
            obj = game_module.Game
            c.screen.set_clip(0,0,800,500)
            c.backgr.set_clip(0,0,800,500)
            game = apply(obj,(c.screen,c.backgr,plug_rcdic,basedir,libdir))# call game constructor
            
            Misc.helptitle = game.helptitle()# call it before translation, we need the org name for the score sheet
            
            # bind gettext to game domain
            try:
                i18n = "gettext.install('%s','%s')" % (plugin_name,localedir)
                exec i18n
            except IOError,info:
                #trace_error()
                print >> sys.stderr, "Can't find i18n files for",stdloc
                print >> sys.stderr, "Using standard, -> english"
                import __builtin__
                __builtin__.__dict__['_'] = gettext.gettext
                            
            c.screen.set_clip()
            c.backgr.set_clip()
            
            print game.__doc__
            print game.start.__doc__
                        
            levels = game.gamelevels
            items = game.gameitems
            
            # stop and score flags
            # stop -> 0=continue, 2=user exit (hit the stopsign), -1=clear screen, 1=exit (game finished)
            game_score = 0
            for l in levels:
                c.display_scorething(str(game_score))
                for i in items: 
                    #print 'i',i
                    stop = 0
                    c.screen.set_clip(0,0,800,500)
                    c.backgr.set_clip(0,0,800,500)
                    game.start(l,i)
                    stop,score = c.mainloop(game)
                    if score and game_score > -1:
                        game_score += score
                        c.display_scorething(str(game_score))# Let the controller do his thing
                        #print 'score',score,'game_score',game_score
                    if stop == -1:
                        stop = 0
                        c.screen.blit(screendim.buffer,(0,0))
                        c.backgr.blit(screendim.buffer,(0,0))
                        screendim.undim()
                    elif stop == 2 or stop == 1:
                        del game
                        # reset counters for placing tux
                        c.tuxplace = 20
                        c.tuxrects = []
                        c.screen.set_clip()
                        c.backgr.set_clip()
                        break
                    c.screen.set_clip()
                    c.backgr.set_clip()
                else:
                    c.place_tux()
                
                if stop == 2 or stop == 1:
                    break
        except MyError,info:
            trace_error()
            print >> sys.stderr,'********************** ERROR *********************\n',\
                    ' A error occured in module',MyError.name,' LineNo:',MyError.line,'\n',\
                    ' The error message is:\n',info,\
                    '\n**************************************************'
            stop = 2
        try:
            gettext.install('childsplay',localedir)
        except IOError:
            import __builtin__
            __builtin__.__dict__['_'] = gettext.gettext
        if stop != 2:
            if game_score:
                c.check_high_score(helptitle,game_score)
                game_score = 0
            c.game_over()
        c.screen.set_clip()
        c.backgr.set_clip()
        c.screen.blit(screendim.buffer,(0,0))
        c.backgr.blit(screendim.buffer,(0,0))
        screendim.undim()
        
        
if __name__ == '__main__':
    ############## Wait until the splash thread is finished ##########
    while splash.thread.isAlive():
        pass
    #################   
    try:
        main()
    finally:
        locale.setlocale(locale.LC_ALL,stdloc)
        
        
        
        